@@ -55,9 +55,8 @@ Please stop compiling the program now
 # define CHAR_BIT 8
 #endif
 
-#if HAVE_INTTYPES_H
 # include <inttypes.h>
-#endif
+
 typedef bool boolean;
 
 #include "regex.h"
@@ -466,7 +465,7 @@ PREDICATEFUNCTION pred_xtype;
 
 
 
-int launch PARAMS((const struct buildcmd_control *ctl,
+int launch PARAMS((struct buildcmd_control *ctl,
 		   struct buildcmd_state *buildstate));
 
 
@@ -515,7 +514,7 @@ boolean apply_predicate(const char *pathname, struct stat *stat_buf, struct pred
 /* find.c. */
 int get_info PARAMS((const char *pathname, struct stat *p, struct predicate *pred_ptr));
 int following_links PARAMS((void));
-int digest_mode PARAMS((mode_t mode, const char *pathname, const char *name, struct stat *pstat, boolean leaf));
+int digest_mode PARAMS((mode_t *mode, const char *pathname, const char *name, struct stat *pstat, boolean leaf));
 boolean default_prints PARAMS((struct predicate *pred));
 boolean looks_like_expression PARAMS((const char *arg, boolean leading));
 
