@@ -38,7 +38,6 @@
 #include "human.h"
 #include "modetype.h"
 #include "filemode.h"
-#include "wait.h"
 #include "printquoted.h"
 #include "buildcmd.h"
 #include "yesno.h"
@@ -66,6 +65,7 @@
 #endif
 
 
+#include <sys/wait.h>
 
 #if HAVE_DIRENT_H
 # include <dirent.h>
@@ -1607,10 +1607,12 @@ pred_prune (const char *pathname, struct stat *stat_buf, struct predicate *pred_
   (void) pathname;
   (void) pred_ptr;
 
-  if (options.do_dir_first == true && /* no effect with -depth */
-      stat_buf != NULL &&
-      S_ISDIR(stat_buf->st_mode))
-    state.stop_at_current_level = true;
+  if (options.do_dir_first == true) { /* no effect with -depth */
+    assert (state.have_stat);
+    if (stat_buf != NULL &&
+	S_ISDIR(stat_buf->st_mode))
+      state.stop_at_current_level = true;
+  }
 
   /* findutils used to return options.do_dir_first here, so that -prune
    * returns true only if -depth is not in effect.   But POSIX requires
@@ -1939,7 +1941,7 @@ prep_child_for_exec (boolean close_stdin, int dir_fd)
 
 
 int
-launch (const struct buildcmd_control *ctl,
+launch (struct buildcmd_control *ctl,
 	struct buildcmd_state *buildstate)
 {
   int wait_status;
