@@ -181,7 +181,7 @@ set_stat_placeholders(struct stat *p)
 
 
 /* Get the stat information for a file, if it is
- * not already known.
+ * not already known.  Returns 0 on success.
  */
 int
 get_statinfo (const char *pathname, const char *name, struct stat *p)
@@ -221,7 +221,7 @@ get_statinfo (const char *pathname, const char *name, struct stat *p)
 }
 
 /* Get the stat/type/inode information for a file, if it is not
- * already known.
+ * already known.   Returns 0 on success (or if we did nothing).
  */
 int
 get_info (const char *pathname,
@@ -263,7 +263,8 @@ get_info (const char *pathname,
       if (result != 0)
 	{
 	  /* Verify some postconditions.  We can't check st_mode for
-	     non-zero-ness because of Savannah bug #16378. */
+	     non-zero-ness because of Savannah bug #16378 (which is
+	     that broken NFS servers can return st_mode==0). */
 	  if (pred_ptr->need_type)
 	    {
 	      assert (state.have_type);
@@ -272,11 +273,16 @@ get_info (const char *pathname,
 	    {
 	      assert (p->st_ino);
 	    }
+	  return -1;		/* failure. */
+	}
+      else
+	{
+	  return 0;		/* success. */
 	}
     }
   else
     {
-      return 0;
+      return 0;			/* success; nothing to do. */
     }
 }
 
@@ -650,7 +656,7 @@ following_links(void)
 /* Take a "mode" indicator and fill in the files of 'state'.
  */
 int
-digest_mode(mode_t mode,
+digest_mode(mode_t *mode,
 	    const char *pathname,
 	    const char *name,
 	    struct stat *pstat,
@@ -659,20 +665,20 @@ digest_mode(mode_t mode,
   /* If we know the type of the directory entry, and it is not a
    * symbolic link, we may be able to avoid a stat() or lstat() call.
    */
-  if (mode)
+  if (*mode)
     {
-      if (S_ISLNK(mode) && following_links())
+      if (S_ISLNK(*mode) && following_links())
 	{
 	  /* mode is wrong because we should have followed the symlink. */
 	  if (get_statinfo(pathname, name, pstat) != 0)
 	    return 0;
-	  mode = state.type = pstat->st_mode;
+	  *mode = state.type = pstat->st_mode;
 	  state.have_type = true;
 	}
       else
 	{
 	  state.have_type = true;
-	  pstat->st_mode = state.type = mode;
+	  pstat->st_mode = state.type = *mode;
 	}
     }
   else
@@ -697,7 +703,7 @@ digest_mode(mode_t mode,
 	   * the mode of the directory entry (S_IFLNK).  Hence now
 	   * that we have the stat information, override "mode".
 	   */
-	  state.type = pstat->st_mode;
+	  state.type = *mode = pstat->st_mode;
 	  state.have_type = true;
 	}
     }
