@@ -25,9 +25,7 @@
 #include <stddef.h>  /* for NULL, in case a nonstandard string.h lacks it */
 
 #include "malloca.h"
-#if HAVE_MBRTOWC
-# include "mbuiter.h"
-#endif
+#include "mbuiter.h"
 
 #define TOLOWER(Ch) (isupper (Ch) ? tolower (Ch) : (Ch))
 
@@ -35,7 +33,6 @@
 #define CANON_ELEMENT(c) TOLOWER (c)
 #include "str-kmp.h"
 
-#if HAVE_MBRTOWC
 /* Knuth-Morris-Pratt algorithm.
    See http://en.wikipedia.org/wiki/Knuth-Morris-Pratt_algorithm
    Return a boolean indicating success:
@@ -192,7 +189,6 @@ knuth_morris_pratt_multibyte (const char *haystack, const char *needle,
   freea (memory);
   return true;
 }
-#endif
 
 /* Find the first occurrence of the character string NEEDLE in the character
    string HAYSTACK, using case-insensitive comparison.
@@ -206,7 +202,6 @@ mbscasestr (const char *haystack, const char *needle)
        - haystack may be very long, and a match of needle found early,
        - needle may be very long, and not even a short initial segment of
          needle may be found in haystack.  */
-#if HAVE_MBRTOWC
   if (MB_CUR_MAX > 1)
     {
       mbui_iterator_t iter_needle;
@@ -319,7 +314,6 @@ mbscasestr (const char *haystack, const char *needle)
 	return (char *) haystack;
     }
   else
-#endif
     {
       if (*needle != '\0')
 	{
