@@ -1,18 +1,18 @@
 #! /bin/sh
 #
 # import-gnulib.sh -- imports a copy of gnulib into findutils
-# Copyright (C) 2003,2004,2005,2006,2007 Free Software Foundation, Inc.
+# Copyright (C) 2003,2004,2005,2006,2007,2009 Free Software Foundation, Inc.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
@@ -31,6 +31,7 @@ unset CDPATH
 
 ## Defaults
 # cvsdir=/doesnotexist
+git_repo="git://git.savannah.gnu.org/gnulib.git"
 configfile="./import-gnulib.config"
 need_checkout=yes
 
@@ -54,8 +55,8 @@ EOF
 
 
 do_checkout () {
-    local cvsdir="$1"
-    echo checking out gnulib from CVS in $cvsdir
+    local gitdir="$1"
+    echo checking out gnulib from GIT in $gitdir
 
     if [ -z "$gnulib_version" ] ; then
 	echo "Error: There should be a gnulib_version setting in $configfile, but there is not." >&2
@@ -63,68 +64,33 @@ do_checkout () {
     fi
 
 
-    if ! [ -d "$cvsdir" ] ; then
-	if mkdir "$cvsdir" ; then
-	echo "Created $cvsdir"
+    if ! [ -d "$gitdir" ] ; then
+	if mkdir "$gitdir" ; then
+	echo "Created $gitdir"
 	else
-	echo "Failed to create $cvsdir" >&2
+	echo "Failed to create $gitdir" >&2
 	exit 1
 	fi
     fi
 
-    # Decide if gnulib_version is probably a date or probably a tag.
-    if date -d yesterday >/dev/null ; then
-	# It looks like GNU date is available
-	if date -d "$gnulib_version" >/dev/null ; then
-	# Looks like a date.
-	cvs_sticky_option="-D"
-	else
-	echo "Warning: assuming $gnulib_version is a CVS tag rather than a date" >&2
-	cvs_sticky_option="-r"
-	fi
-    else
-	# GNU date unavailable, assume the version is a date
-	cvs_sticky_option="-D"
-    fi
-
-
-
     (
-	# Change directory unconditionally (rater than using checkout
-	# -d) so that cvs does not pick up defaults from ./CVS.  Those
-	# defaults refer to our own CVS repository for our code, not
-	# to gnulib.
-	cd $cvsdir
+	# Change directory unconditionally.  We used to do this to avoid
+	# the cvs client picking up defaults from findutils' ./CVS/*, but
+	# now we just do it for the sake of a minimum change.
+	cd $gitdir
 
-	# gnulib now uses git as master repository, but used to use
-	# CVS.   Check that we are not running against an old working
-	# directory which is still pointing at the old CVS repository.
-	rootfile=gnulib/CVS/Root
-	cvs_git_root=":pserver:anonymous@pserver.git.sv.gnu.org:/gnulib.git"
-
-	if test -d gnulib/CVS
-	then
-	  if test x"$(cat $rootfile)" == x"$cvs_git_root"; then
-	      echo "Using the git repository via git-cvs-pserver..."
-	  else
-	      echo "WARNING: Migrating from old CVS repository" >&2
-	      # Force use of "cvs checkout" as opposed to update.
-	      mv gnulib gnulib.before-git-migration
-	  fi
-	fi
-
-	if test -d gnulib/CVS ; then
-	  cd gnulib
-	  cmd=update
-	  root="" # use previous
-	  args=
+	if test -d gnulib/.git ; then
+	  echo "Git repository was already initialised."
 	else
-	  cmd=checkout
-	  root="-d $cvs_git_root"
-	  args="-d gnulib HEAD"
+	  echo "Cloning the git repository..."
+	  # In the future we may use a shallow clone to
+	  # save bandwidth.
+	  git clone "$git_repo"
 	fi
+	cd gnulib
 	set -x
-	cvs -q $root $cmd $cvs_sticky_option "$gnulib_version" $args
+	git fetch origin
+	git checkout "$gnulib_version"
 	set +x
     )
 }
@@ -171,16 +137,18 @@ run_gnulib_tool() {
 }
 
 rehack() {
-    echo "Updating the license of $1"
+    printf "Updating the license of $1... "
     # Use cp to get the permissions right first
     cp -fp "$1" "$1".new
     sed -e  \
 's/Free Software Foundation\([;,]\) either version [2]/Free Software Foundation\1 either version 3/' < "$1" > "$1".new
     if cmp "$1" "$1".new >/dev/null
     then
+	echo "no change needed."
 	rm -f "$1".new
     else
 	rm -f "$1" && mv "$1".new "$1"
+	echo "done."
     fi
 }
 
@@ -190,17 +158,16 @@ copyhack() {
     src="$1"
     dst="$2"
     shift 2
-    if test -d "$dst" 
+    if test -d "$dst"
     then
 	dst="$dst"/"$(basename $src)"
     fi
     cp -fp "$src" "$dst" && rehack "$dst"
-
 }
 
 
 update_licenses() {
-    for f in $gpl3_update_files 
+    for f in $gpl3_update_files
     do
       rehack "$f" || exit
     done
@@ -217,12 +184,9 @@ hack_gnulib_tool_output() {
       esac
       copyhack "${gnulibdir}"/"$file" "$dest" || exit
     done
-    
-
-
 
     cat > gnulib/Makefile.am <<EOF
-# Copyright (C) 2004 Free Software Foundation, Inc.
+# Copyright (C) 2004, 2009 Free Software Foundation, Inc.
 #
 # This file is free software, distributed under the terms of the GNU
 # General Public License.  As a special exception to the GNU General
@@ -233,15 +197,17 @@ hack_gnulib_tool_output() {
 # This file was generated by $0 $original_cmd_line_args.
 #
 SUBDIRS = lib
+EXTRA_DIST = m4/gnulib-cache.m4
 EOF
 }
 
 
 refresh_output_files() {
+    autopoint -f &&
     aclocal -I m4 -I gnulib/m4     &&
     autoheader                     &&
     autoconf                       &&
-    automake --add-missing --copy 
+    automake --add-missing --copy
 }
 
 
@@ -269,6 +235,94 @@ EOF
 }
 
 
+check_merge_driver() {
+    local config_file=".git/config"
+    fixmsg="
+
+We recommend that you use a git merge driver for the ChangeLog file.
+This simplifies the task of merging branches.
+Please see the README section in gnulib-git/gnulib/lib/git-merge-changelog.c
+
+If you've read that and don't want to use it, just set the environment variable
+DO_NOT_WANT_CHANGELOG_DRIVER.
+
+Example:
+  git config merge.merge-changelog.name 'GNU-style ChangeLog merge driver'
+  git config merge.merge-changelog.driver /usr/local/bin/git-merge-changelog  %O %A %B
+  echo 'ChangeLog    merge=merge-changelog' >> .gitattributes
+"
+    if [[ -z "$DO_NOT_WANT_CHANGELOG_DRIVER" ]] ; then
+	if git branch | egrep -q '\* *(master|rel-)'; then
+	# We are on the master branch or a release branch.
+	# Perhaps the user is simply building from git sources.
+	# Issue our message as a warning rather than an error.
+	    fatal=false
+	    label="Warning"
+	else
+	    fatal=true
+	    label="ERROR"
+	fi
+    else
+	fatal=false
+	label="Warning"
+    fi
+    if git config --get  merge.merge-changelog.name >/dev/null ; then
+	driver="$(git config --get merge.merge-changelog.driver |
+                  sed -e 's/[ 	].*//')"
+	if [[ $? -eq 0 ]]; then
+	    if ! [[ -x "$driver" ]]; then
+		echo "ERROR: Merge driver $driver is not executable." >&2
+		echo "ERROR: Please fix $config_file or install $driver" >&2
+		# Always fatal - if configured, the merge driver should work.
+		exit 1
+	    else
+		if [[ -f .gitattributes ]] ; then
+		    echo "The ChangeLog merge driver configuration seems OK."
+		else
+		    echo "$label"': you have no .gitattributes file' >&2
+		    echo "$fixmsg" >&2
+		    if $fatal; then
+			exit 1
+		    fi
+		fi
+	    fi
+	else
+	    echo "$label"': There is no driver specified in [merge "merge-changelog"] in' "$config_file" >&2
+	    echo "$fixmsg" >&2
+	    if $fatal; then
+		exit 1
+	    fi
+	fi
+    else
+	echo "$label"': There is no name specified in [merge "merge-changelog"] in' "$config_file" >&2
+	echo "$fixmsg" >&2
+	if $fatal; then
+	    exit 1
+	fi
+    fi
+}
+
+
+move_cvsdir() {
+    local cvs_git_root=":pserver:anonymous@pserver.git.sv.gnu.org:/gnulib.git"
+
+    if test -d gnulib-cvs/gnulib/CVS
+    then
+      if test x"$(cat gnulib-cvs/gnulib/CVS/Root)" == x"$cvs_git_root"; then
+          # We cannot use the git-cvspserver interface because
+          # "update -D" doesn't work.
+          echo "WARNING: Migrating from git-cvs-pserver to native git..." >&2
+          savedir=gnulib-cvs.before-nativegit-migration
+      else
+          # The old CVS repository is not updated any more.
+          echo "WARNING: Migrating from old CVS repository to native git" >&2
+          savedir=gnulib-cvs.before-git-migration
+      fi
+      mv gnulib-cvs $savedir || exit 1
+      echo "Please delete $savedir eventually"
+    fi
+}
+
 main() {
     ## Option parsing
     local gnulibdir=/doesnotexist
@@ -285,10 +339,19 @@ main() {
     # to use, even if we don't want to know the CVS version.
     . $configfile || exit 1
 
-    ## If -d was not given, do CVS checkout/update
+    ## If -d was not given, do update
     if [ $need_checkout = yes ] ; then
-	do_checkout gnulib-cvs
-	gnulibdir=gnulib-cvs/gnulib
+	if ! git version > /dev/null; then
+	    cat >&2 <<EOF
+You now need the tool 'git' in order to check out the correct version
+of the gnulib code.  See http://git.or.cz/ for more information about git.
+EOF
+	    exit 1
+	fi
+	move_cvsdir
+	do_checkout gnulib-git
+	check_merge_driver
+	gnulibdir=gnulib-git/gnulib
     else
 	echo "Warning: using gnulib code which already exists in $gnulibdir" >&2
     fi
@@ -300,7 +363,8 @@ main() {
 	hack_gnulib_tool_output "${gnulibdir}" &&
 	refresh_output_files &&
 	update_licenses &&
-	update_version_file   ; then
+	update_version_file
+    then
 	echo Done.
     else
 	echo FAILED >&2
